﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Threading;
using System.ComponentModel;

namespace PaginatedUltraWinGrid
{
    public sealed class DataProvider
    {

        #region Events

            public event EventHandler<DataEventArgs> DataProviding;
            private void dataProvide(object data)
            {
                if (this.DataProviding!=null)
                    this.DataProviding(this, new DataEventArgs(data as DataTable));
            }

        #endregion


        #region Properties

            private const int VectorSize = 10000;
            private int rowsCount = -1;
            public int RowsCount {
                get
                {
                    if (this.rowsCount == -1)
                        return VectorSize;
                    else
                        return this.rowsCount;
                }
                internal set
                {
                    this.rowsCount = value;
                }
            }

        #endregion


        #region Fields

            DataTable tableUsers = new DataTable("users");
            private BackgroundWorker worker = new BackgroundWorker();

        #endregion


        public DataProvider()
        {
            this.worker.DoWork += new DoWorkEventHandler(worker_DoWork);
            this.worker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(worker_RunWorkerCompleted);
        }

        void worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            this.dataProvide(e.Result);
        }


        void worker_DoWork(object sender, DoWorkEventArgs e)
        {
            string filter = ((object[])e.Argument)[0] as string;
            string sort = ((object[])e.Argument)[1] as string;
            int pos = (int)((object[])e.Argument)[2];
            int length = (int)((object[])e.Argument)[3];

            DataRow[] finalRows = this.tableUsers.Select(filter, sort);

            if (pos > finalRows.Count() - 1) {
                e.Result = null;
                return;
            }

            this.RowsCount = finalRows.Count();

            DataTable table = this.tableUsers.Clone();
            for (int i = pos; i < (length + pos); i++) {
                Thread.Sleep(1);
                if (i > finalRows.Count() - 1)
                    break;
                table.ImportRow(finalRows[i]);
            }
            e.Result = table;
        }


        public void GenerateData()
        {
            tableUsers.Columns.Add("Index", typeof(int));
            tableUsers.Columns["Index"].Caption = "Index";

            tableUsers.Columns.Add("FirstName", typeof(string));
            tableUsers.Columns["FirstName"].Caption = "First Name";

            tableUsers.Columns.Add("LastName", typeof(string));
            tableUsers.Columns["LastName"].Caption = "Last Name";

            tableUsers.Columns.Add("Age", typeof(int));
            tableUsers.Columns["Age"].Caption = "Age";

            for (int i=0; i<VectorSize;i++){
                DataRow row = tableUsers.NewRow();
                row["Index"] = i+1;
                row["FirstName"] = this.GenerateName();
                row["LastName"] = this.GenerateName();
                row["Age"] = r.Next(82)+18;
                this.tableUsers.Rows.Add(row);
            }
        }


        public void RequestUsers(string filter, string sort, int pos, int length)
        {
            object[] args = new object[] { filter, sort, pos, length };

            if (!this.worker.IsBusy)
                this.worker.RunWorkerAsync(args);
        }

        public IEnumerable<object> DistinctValues(string columnKey, string filter)
        {
            DataRow[] rows = this.tableUsers.Select(filter);
            return rows.Select(x => x[columnKey]).Distinct();
        }


        #region Name Generator

            Random r = new Random();
            string alphabetC = "bcdfglmnprstvz";
            string alphabetV = "aeiou";

            string GenerateName()
            {
                Func<char> randomLetterC = () => alphabetC[r.Next(alphabetC.Length)];
                Func<char> randomLetterV = () => alphabetV[r.Next(alphabetV.Length)];
                Func<int, string> makeName =
                  (length) => new string(Enumerable.Range(0, length)
                     .Select(x => x == 0 ?
                         char.ToUpper(randomLetterC()) :
                         x % 2 == 0 ? randomLetterC() : randomLetterV())
                     .ToArray());
                string first = makeName(r.Next(5) + 5);
                //string last = makeName(r.Next(7) + 7);
                //string company = makeName(r.Next(7) + 7) + " Inc.";
                return first;
            }

        #endregion

    }


    public class DataEventArgs: EventArgs
    {
        public DataTable Data{get;set;}

        public DataEventArgs(DataTable data)
        {
            this.Data = data;
        }
    }

}
